unless Object.const_defined?('Freshen')
  raise "You can't load this file. Please load the freshen file instead."
end

class Freshen
  # A RuntimeError subclass for when the current
  # system's kernel is unsupported.
  #
  class UnsupportedKernel < RuntimeError
    attr_reader :kernel
    
    def initialize(kernel)
      @kernel = kernel
    end
    
    def to_s
      "Freshen doesn't support a #{@kernel} kernel"
    end
  end
  
  # A NotImplementedError subclass for when the
  # update method of a freshener subclass has not
  # been implemented.
  #
  class UpdateMethodNotImplementedError < NotImplementedError
    attr_reader :name
    
    def initialize(freshener)
      @name = freshener.class.name.split('::').last
    end
    
    def to_s
      "#{@name} does not implement the update method"
    end
  end
  
  # An Exception subclass for when a freshener
  # is not installed.
  #
  class FreshenerNotInstalled < Exception
    attr_reader :name
    
    def initialize(name)
      @name = name
    end
    
    def to_s
      "The freshener #{@name} is not installed"
    end
  end
  
  # A LoadError subclass for when a freshener
  # failed to load.
  #
  class FreshenerLoadError < LoadError
    attr_reader :name
    
    def initialize(name)
      @name = name
    end
    
    def to_s
      "Unable to load freshener #{@name}"
    end
  end
  
  # A RuntimeError subclass for when an executing
  # command fails.
  #
  class ExecutableError < RuntimeError
    attr_reader :name, :command, :args
    
    def initialize(name, command, *args)
      @name = name
      @command = command
      @args = args.map { |a| a.to_s }
    end
    
    def to_s
      command = Executable.command_string(@command, *@args)
      
      "#{@name} freshener failed command '#{command}'"
    end
  end
end
